"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = upcomingTripRenderer;

var _react = _interopRequireDefault(require("react"));

var _reactIntl = require("react-intl");

var _formattedDuration = _interopRequireDefault(require("../../../util/formatted-duration"));

var _itinerary = require("../../../../util/itinerary");

var _viewer = require("../../../../util/viewer");

var _baseRenderer = _interopRequireDefault(require("./base-renderer"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Calculates various data for monitored trips that are going to happen in the
 * future. Some realtime data might already exist when these trips are within
 * their lead time, so additional calculations and output occurs in those cases.
 */
function upcomingTripRenderer({
  dateFormat,
  monitoredTrip,
  onTimeThresholdSeconds,
  timeFormat
}) {
  const data = (0, _baseRenderer.default)(monitoredTrip);
  const ONE_MINUTE = 60 * 1000;
  const minutesUntilTripStart = (0, _itinerary.getMinutesUntilItineraryStart)(data.matchingItinerary); // set default panel bootstrap style

  data.panelBsStyle = 'info';
  const tripStart = data.matchingItinerary.startTime;
  const monitoringStart = tripStart - monitoredTrip.leadTimeInMinutes * ONE_MINUTE;
  data.bodyText = /*#__PURE__*/_react.default.createElement(_reactIntl.FormattedMessage, {
    id: "components.TripStatusRenderers.upcoming.tripBegins",
    values: {
      monitoringStart: monitoringStart,
      tripStart: tripStart
    }
  }); // set heading text and whether to show the alerts based on whether the
  // minutes until the trip starts is within the trip's lead time

  if (minutesUntilTripStart > monitoredTrip.leadTimeInMinutes) {
    // trip is not yet within lead time
    data.panelBsStyle = undefined;
    data.headingText = /*#__PURE__*/_react.default.createElement(_reactIntl.FormattedMessage, {
      id: "components.TripStatusRenderers.upcoming.nextTripBegins",
      values: {
        tripDatetime: tripStart
      }
    }); // Do not render alerts if the lead time hasn't passed yet. If there are
    // alerts in the upcoming trip, it is possible that these alerts are now
    // stale due to the trip not having been calculated in a while. Therefore,
    // they shouldn't be shown until after the lead time.

    data.shouldRenderAlerts = false;
  } else {
    // trip is within lead time
    // analyze whether the journey state indicates that the matching itinerary
    // has realtime data. It is assumed that realtime data will exist for
    // itineraries that have differing values of the scheduled departure time
    // and matching itinerary departure time.
    if (data.journeyState.hasRealtimeData) {
      // calculate the deviation from the scheduled departure time (positive
      // value indicates delay)
      const departureDeviationSeconds = (data.matchingItinerary.startTime - data.journeyState.scheduledDepartureTimeEpochMillis) / 1000;
      const absDeviation = Math.abs(departureDeviationSeconds);
      const tripStatus = (0, _viewer.getTripStatus)(true, departureDeviationSeconds, onTimeThresholdSeconds);

      if (tripStatus === _viewer.REALTIME_STATUS.ON_TIME) {
        // about on time
        data.headingText = /*#__PURE__*/_react.default.createElement(_reactIntl.FormattedMessage, {
          id: "components.TripStatusRenderers.upcoming.tripStartsSoonOnTime"
        });
        data.panelBsStyle = 'success';
      } else if (tripStatus === _viewer.REALTIME_STATUS.LATE) {
        // delayed
        data.headingText = /*#__PURE__*/_react.default.createElement(_reactIntl.FormattedMessage, {
          id: "components.TripStatusRenderers.upcoming.tripStartIsDelayed",
          values: {
            duration: /*#__PURE__*/_react.default.createElement(_formattedDuration.default, {
              values: absDeviation
            })
          }
        });
        data.panelBsStyle = 'warning';
      } else {
        // early
        data.headingText = /*#__PURE__*/_react.default.createElement(_reactIntl.FormattedMessage, {
          id: "components.TripStatusRenderers.upcoming.tripStartIsEarly",
          values: {
            duration: /*#__PURE__*/_react.default.createElement(_formattedDuration.default, {
              values: absDeviation
            })
          }
        });
        data.panelBsStyle = 'warning';
      }
    } else {
      data.headingText = /*#__PURE__*/_react.default.createElement(_reactIntl.FormattedMessage, {
        id: "components.TripStatusRenderers.upcoming.tripStartsSoonNoUpdates"
      });
    }
  }

  data.shouldRenderTogglePauseTripButton = true;
  data.shouldRenderToggleSnoozeTripButton = true;
  return data;
}

module.exports = exports.default;

//# sourceMappingURL=upcoming-trip-renderer.js